==> parser-2.9.10-20251125.172435-3-sources.jar: org/mule/weave/v2/api/tooling/internal/DefaultDWAstNode.scala <==
package org.mule.weave.v2.api.tooling.internal

import org.mule.weave.v2.api.tooling.ast.DWAstNode
import org.mule.weave.v2.api.tooling.location.Location
import org.mule.weave.v2.api.tooling.ts.DWType
import org.mule.weave.v2.codegen.CodeGenerator
import org.mule.weave.v2.parser.ast.AstNode
import org.mule.weave.v2.parser.ast.LiteralValueAstNode
import org.mule.weave.v2.parser.ast.variables.NameIdentifier
import org.mule.weave.v2.scope.AstNavigator
import org.mule.weave.v2.scope.ScopesNavigator
import org.mule.weave.v2.ts.TypeGraph
import org.mule.weave.v2.utils.Optionals.toJavaOptional

import java.util.Optional
import scala.language.postfixOps

case class DefaultDWAstNode(
  node: AstNode,
  astNavigator: AstNavigator,
  typeGraph: Option[TypeGraph],
  scopeGraph: Option[ScopesNavigator])
    extends DWAstNode {

  override def getReference: Optional[DWAstNode] = {
    val maybeWrapper: Option[DWAstNode] = scopeGraph
      .flatMap(_.resolveReference(node))
      .map(astNode => DefaultDWAstNode(astNode, astNavigator, typeGraph, scopeGraph))
    maybeWrapper asJava
  }

  override def getTypeOf: Optional[DWType] = {
    val maybeWrapper: Option[DWType] = typeGraph
      .flatMap(tgraph => tgraph.findNode(node))
      .flatMap(_.resultType())
    maybeWrapper asJava
  }

  override def getChildren: Array[DWAstNode] = {
    node.children().map(astNode => DefaultDWAstNode(astNode, astNavigator, typeGraph, scopeGraph)).toArray
  }

  override def getKind: String = {
    node.getKind()
  }

  override def getValue: Optional[String] = {
    node match {
      case ni: NameIdentifier      => Optional.of(ni.fullQualifiedName())
      case lv: LiteralValueAstNode => Optional.of(lv.literalValue)
      case _                       => Optional.empty()
    }
  }

  override def emitCode(): String = {
    CodeGenerator.generate(node)
  }

  override def getLocation: Location = {
    node.location()
  }

  override def getDocumentation: Optional[DWAstNode] = {
    Optional.ofNullable(node.weaveDoc.map(weaveDocNode => DefaultDWAstNode(weaveDocNode, astNavigator, typeGraph, scopeGraph)).orNull)
  }

  override def getParent: Optional[DWAstNode] = {
    val maybeWrapper: Option[DWAstNode] = astNavigator
      .parentOf(node)
      .map(astNode => DefaultDWAstNode(astNode, astNavigator, typeGraph, scopeGraph))
    maybeWrapper asJava
  }
}

==> parser-2.11.0-20251120.154239-58-sources.jar: org/mule/weave/v2/api/tooling/internal/DefaultDWAstNode.scala <==
package org.mule.weave.v2.api.tooling.internal

import org.mule.weave.v2.api.tooling.ast.DWAstNode
import org.mule.weave.v2.api.tooling.location.Location
import org.mule.weave.v2.api.tooling.ts.DWType
import org.mule.weave.v2.codegen.CodeGenerator
import org.mule.weave.v2.parser.ast.AstNode
import org.mule.weave.v2.parser.ast.LiteralValueAstNode
import org.mule.weave.v2.parser.ast.structure.NamespaceNode
import org.mule.weave.v2.parser.ast.types.TypeReferenceNode
import org.mule.weave.v2.parser.ast.variables.NameIdentifier
import org.mule.weave.v2.parser.ast.variables.VariableReferenceNode
import org.mule.weave.v2.scope.AstNavigator
import org.mule.weave.v2.scope.ScopesNavigator
import org.mule.weave.v2.ts.TypeGraph
import org.mule.weave.v2.utils.Optionals.toJavaOptional

import java.util.Optional
import scala.language.postfixOps

case class DefaultDWAstNode(
  node: AstNode,
  astNavigator: AstNavigator,
  typeGraph: Option[TypeGraph],
  scopeGraph: Option[ScopesNavigator])
    extends DWAstNode {

  override def getReference: Optional[DWAstNode] = {
    val maybeIdentifier = node match {
      case vrn: VariableReferenceNode => Some(vrn.variable)
      case nsn: NamespaceNode         => Some(nsn.nameIdentifier)
      case trn: TypeReferenceNode     => Some(trn.variable)
      case ni: NameIdentifier         => Some(ni)
      case _                          => None
    }

    val maybeWrapper: Option[DWAstNode] = maybeIdentifier.flatMap(id => {
      scopeGraph
        .flatMap(_.resolveVariable(id))
        .map(reference => {
          // If the reference is cross module we don't have a way here to get the relevant type graph
          val newTypeGraph = if (reference.isCrossModule) None else typeGraph

          DefaultDWAstNode(
            reference.referencedNode,
            reference.scope.astNavigator(),
            newTypeGraph,
            Some(reference.scope.scopesNavigator()))
        })
    })

    maybeWrapper asJava
  }

  override def getTypeOf: Optional[DWType] = {
    val maybeWrapper: Option[DWType] = typeGraph
      .flatMap(tgraph => tgraph.findNode(node))
      .flatMap(_.resultType())
    maybeWrapper asJava
  }

  override def getChildren: Array[DWAstNode] = {
    node.children().map(astNode => DefaultDWAstNode(astNode, astNavigator, typeGraph, scopeGraph)).toArray
  }

  override def getKind: String = {
    node.getKind()
  }

  override def getValue: Optional[String] = {
    node match {
      case ni: NameIdentifier      => Optional.of(ni.fullQualifiedName())
      case lv: LiteralValueAstNode => Optional.of(lv.literalValue)
      case _                       => Optional.empty()
    }
  }

  override def emitCode(): String = {
    CodeGenerator.generate(node)
  }

  override def getLocation: Location = {
    node.location()
  }

  override def getDocumentation: Optional[DWAstNode] = {
    Optional.ofNullable(node.weaveDoc.map(weaveDocNode => DefaultDWAstNode(weaveDocNode, astNavigator, typeGraph, scopeGraph)).orNull)
  }

  override def getParent: Optional[DWAstNode] = {
    val maybeWrapper: Option[DWAstNode] = astNavigator
      .parentOf(node)
      .map(astNode => DefaultDWAstNode(astNode, astNavigator, typeGraph, scopeGraph))
    maybeWrapper asJava
  }
}
